Package: rankdifferencetest
Title: Kornbrot's Rank Difference Test
Version: 2025.12.4
Authors@R: person("Brett", "Klamer", 
                  email = "code@brettklamer.com", 
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0002-8334-4831"))
Description: Implements Kornbrot's rank difference test as described in 
    <doi:10.1111/j.2044-8317.1990.tb00939.x>. This method is a modified 
    Wilcoxon signed-rank test which produces consistent and meaningful 
    results for ordinal or monotonically-transformed data.
URL: https://brettklamer.com/work/rankdifferencetest,
        https://bitbucket.org/bklamer/rankdifferencetest
License: GPL-3
LazyData: TRUE
Depends: R (>= 4.3.0)
Imports: Rdpack, stats, exactRankTests, Hmisc, boot, generics
Suggests: tinytest, rmarkdown, tibble, coin
RdMacros: Rdpack
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-12-05 04:36:40 UTC; x
Author: Brett Klamer [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8334-4831>)
Maintainer: Brett Klamer <code@brettklamer.com>
Repository: CRAN
Date/Publication: 2025-12-05 04:50:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:50:03 UTC; unix
