# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,pmedian)
S3method(as.data.frame,rdpmedian)
S3method(as.data.frame,rdt)
S3method(as.data.frame,srt)
S3method(print,pmedian)
S3method(print,rdpmedian)
S3method(print,rdt)
S3method(print,srt)
S3method(rdt_data,data.frame)
S3method(rdt_data,numeric)
S3method(srt_data,data.frame)
S3method(srt_data,numeric)
S3method(srt_pmedian,asymptotic)
S3method(srt_pmedian,bootstrap)
S3method(srt_pmedian,none)
S3method(srt_pmedian,shift)
S3method(srt_pmedian,wilcoxon)
S3method(srt_pvalue,asymptotic)
S3method(srt_pvalue,shift)
S3method(srt_pvalue,wilcoxon)
S3method(srt_statistic,asymptotic)
S3method(srt_statistic,shift)
S3method(srt_statistic,wilcoxon)
S3method(tidy,pmedian)
S3method(tidy,rdpmedian)
S3method(tidy,rdt)
S3method(tidy,srt)
export(pmedian)
export(pmedian2)
export(rdpmedian)
export(rdpmedian2)
export(rdt)
export(rdt2)
export(rdt_data)
export(srt)
export(srt2)
export(srt_data)
export(srt_pmedian)
export(srt_pvalue)
export(srt_result)
export(srt_statistic)
export(tidy)
importFrom(Hmisc,pMedian)
importFrom(Rdpack,reprompt)
importFrom(boot,boot)
importFrom(boot,boot.ci)
importFrom(exactRankTests,pperm)
importFrom(exactRankTests,qperm)
importFrom(generics,tidy)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,psignrank)
importFrom(stats,qnorm)
importFrom(stats,qsignrank)
importFrom(stats,reshape)
importFrom(stats,setNames)
importFrom(stats,terms.formula)
importFrom(stats,uniroot)
